@@ -1,44 +1,41 @@
+# mypy: allow-untyped-defs
 """Support for providing temporary directories to test functions."""
+
+from __future__ import annotations
+
 import dataclasses
 import os
-import re
-import tempfile
 from pathlib import Path
+import re
 from shutil import rmtree
+import tempfile
 from typing import Any
 from typing import Dict
+from typing import final
 from typing import Generator
-from typing import Optional
-from typing import TYPE_CHECKING
-from typing import Union
-
-from _pytest.nodes import Item
-from _pytest.reports import CollectReport
-from _pytest.stash import StashKey
-
-if TYPE_CHECKING:
-    from typing_extensions import Literal
-
-    RetentionType = Literal["all", "failed", "none"]
-
-
-from _pytest.config.argparsing import Parser
+from typing import Literal
 
+from .pathlib import cleanup_dead_symlinks
 from .pathlib import LOCK_TIMEOUT
 from .pathlib import make_numbered_dir
 from .pathlib import make_numbered_dir_with_cleanup
 from .pathlib import rm_rf
-from .pathlib import cleanup_dead_symlinks
-from _pytest.compat import final, get_user_id
+from _pytest.compat import get_user_id
 from _pytest.config import Config
 from _pytest.config import ExitCode
 from _pytest.config import hookimpl
+from _pytest.config.argparsing import Parser
 from _pytest.deprecated import check_ispytest
 from _pytest.fixtures import fixture
 from _pytest.fixtures import FixtureRequest
 from _pytest.monkeypatch import MonkeyPatch
+from _pytest.nodes import Item
+from _pytest.reports import TestReport
+from _pytest.stash import StashKey
+
 
 tmppath_result_key = StashKey[Dict[str, bool]]()
+RetentionType = Literal["all", "failed", "none"]
 
 
 @final
@@ -49,20 +46,20 @@ class TempPathFactory:
     The base directory can be configured using the ``--basetemp`` option.
     """
 
-    _given_basetemp: Optional[Path]
+    _given_basetemp: Path | None
     # pluggy TagTracerSub, not currently exposed, so Any.
     _trace: Any
-    _basetemp: Optional[Path]
+    _basetemp: Path | None
     _retention_count: int
-    _retention_policy: "RetentionType"
+    _retention_policy: RetentionType
 
     def __init__(
         self,
-        given_basetemp: Optional[Path],
+        given_basetemp: Path | None,
         retention_count: int,
-        retention_policy: "RetentionType",
+        retention_policy: RetentionType,
         trace,
-        basetemp: Optional[Path] = None,
+        basetemp: Path | None = None,
         *,
         _ispytest: bool = False,
     ) -> None:
@@ -85,7 +82,7 @@ class TempPathFactory:
         config: Config,
         *,
         _ispytest: bool = False,
-    ) -> "TempPathFactory":
+    ) -> TempPathFactory:
         """Create a factory according to pytest configuration.
 
         :meta private:
@@ -201,7 +198,7 @@ class TempPathFactory:
         return basetemp
 
 
-def get_user() -> Optional[str]:
+def get_user() -> str | None:
     """Return the current user name, or None if getuser() does not work
     in the current environment (see #1010)."""
     try:
@@ -209,7 +206,7 @@ def get_user() -> Optional[str]:
         import getpass
 
         return getpass.getuser()
-    except (ImportError, KeyError):
+    except (ImportError, OSError, KeyError):
         return None
 
 
@@ -259,7 +256,7 @@ def _mk_tmp(request: FixtureRequest, factory: TempPathFactory) -> Path:
 @fixture
 def tmp_path(
     request: FixtureRequest, tmp_path_factory: TempPathFactory
-) -> Generator[Path, None, None]:
+) -> Generator[Path]:
     """Return a temporary directory path object which is unique to each test
     function invocation, created as a sub directory of the base temporary
     directory.
@@ -268,12 +265,11 @@ def tmp_path(
     and old bases are removed after 3 sessions, to aid in debugging.
     This behavior can be configured with :confval:`tmp_path_retention_count` and
     :confval:`tmp_path_retention_policy`.
-    If ``--basetemp`` is used then it is cleared each session. See :ref:`base
-    temporary directory`.
+    If ``--basetemp`` is used then it is cleared each session. See
+    :ref:`temporary directory location and retention`.
 
     The returned object is a :class:`pathlib.Path` object.
     """
-
     path = _mk_tmp(request, tmp_path_factory)
     yield path
 
@@ -290,7 +286,7 @@ def tmp_path(
     del request.node.stash[tmppath_result_key]
 
 
-def pytest_sessionfinish(session, exitstatus: Union[int, ExitCode]):
+def pytest_sessionfinish(session, exitstatus: int | ExitCode):
     """After each session, remove base directory if all the tests passed,
     the policy is "failed", and the basetemp is not specified by a user.
     """
@@ -315,10 +311,12 @@ def pytest_sessionfinish(session, exitstatus: Union[int, ExitCode]):
         cleanup_dead_symlinks(basetemp)
 
 
-@hookimpl(tryfirst=True, hookwrapper=True)
-def pytest_runtest_makereport(item: Item, call):
-    outcome = yield
-    result: CollectReport = outcome.get_result()
-
-    empty: Dict[str, bool] = {}
-    item.stash.setdefault(tmppath_result_key, empty)[result.when] = result.passed
+@hookimpl(wrapper=True, tryfirst=True)
+def pytest_runtest_makereport(
+    item: Item, call
+) -> Generator[None, TestReport, TestReport]:
+    rep = yield
+    assert rep.when is not None
+    empty: dict[str, bool] = {}
+    item.stash.setdefault(tmppath_result_key, empty)[rep.when] = rep.passed
+    return rep
